"""
Example on how to use the rendering module
"""
# Import module from blender
import os
import tempfile


environment_name = 'toolbox-navigation'


def activate_virtualenv(name):
    """Activate given virtualenv.
    Virtualenv home folder is given by environment variable
    ``WORKON_HOME`` or ``~/Envs`.
    """
    if 'WORKON_HOME' in os.environ:
        home = os.environ['WORKON_HOME']
    else:
        home = os.path.expanduser(os.path.join('~', 'Envs'))
    filepath = os.path.join(home, name, 'bin', 'activate_this.py')
    with open(filepath, 'r') as f:
        exec(f.read(), dict(__file__=filepath))


activate_virtualenv(environment_name)
#
# Import module in virtualenv
#
import numpy as np  # noqa: E402
from navipy.rendering.bee_sampling import BeeSampling  # noqa: E402

# create a bee sampling
bee_samp = BeeSampling()
# Create a list of point from which we want to render images
# Careful three d meshing grow quickly.
x = np.linspace(-25, 25, 201)
y = np.linspace(-25, 25, 201)
z = np.linspace(1.8, 1.8, 1)
alpha_1 = np.array([0]) + np.pi / 2
alpha_2 = np.array([0])
alpha_3 = np.array([0])
bee_samp.create_sampling_grid(
    x, y, z, alpha1=alpha_1, alpha2=alpha_2, alpha3=alpha_3)
# Assign maximum world dimension. Otherwise the distance
# will go to infinity, and the distance to objects after compression
# will be set to 0
world_dim = 50 * np.sqrt(2)
bee_samp.world_dim = world_dim
# set the number of sample for rendring in blender
bee_samp.set_cycle_samples(samples=50)
# Rendering in a tmp folder
with tempfile.TemporaryDirectory() as folder:
    bee_samp.render(folder + '/database.db')
